open Useful

let help () =
  View.text_page
"
           funLife help
           
         Keyboard controls
               -----

        / backward: undo            Back space
One step                            Space bar
        \\ forward: redo             Tab

Limit existing cells to the screen  L

Run                                 R
Stop                                Lot of keys
Speed                               0, 1,... ..., 9

      / colour under mouse cursor   C
Change  borders of cells            B
      \\ shape  /        \\           S
       
Zoom  in                            +
    \\ out                           -

Help                                ? or H

Quit                                Q
 
Clear                               K

           Mouse controls
               -----
Toggle cell              No move click
Drag view                Click and move

"

let stop_or_control f =
  if !AutoPlay.running then
    AutoPlay.stop ()
  else
    f ()

let key k =
  match Char.uppercase k with
    
    ' ' ->
      stop_or_control Automaton.step

  | 'L' ->
      stop_or_control Automaton.limit

  | '\008' -> (* Back Space *) 
      stop_or_control WorldHistory.undo

  | '\009' -> (* Tab *) 
      stop_or_control WorldHistory.redo
    
  | 'K' ->
      stop_or_control WorldHistory.clear

  | 'A' ->
      stop_or_control View.random

  | 'R' ->
    AutoPlay.run ()
    
  | '0' .. '9' as s -> 
    AutoPlay.set_speed (int_of_char s - 48);

  | 'C' ->
    View.change_colour ()

  | 'B' ->
    View.change_borders ()

  | 'S' ->
    View.change_shape ()
    
  | '+' ->
    View.scale ( * )
  | '-' ->
    View.scale (/)

  | '?' | 'H' ->
    help ()

  | 'Q' ->
    raise Exit

  | _ ->
    AutoPlay.stop ()

let _ =
  WorldHistory.step := Automaton.map_array_set_step;
  let args = Sys.argv in
    (match Array.length args with
      2 ->
        Io.try_load WorldHistory.init args.(1)
    | 1 ->
        (try
          Io.try_load WorldHistory.init "current.l"
        with Sys_error "current.l: No such file or directory" ->
          ())
    | _ ->
        Printf.printf "Usage: %s [pattern_file]\n" args.(0);
        exit 0);
(*  Automaton.init_test ();*)
  
    View.draw ();
(*  AutoPlay.set_speed (9);
    AutoPlay.run ();*)
    ignore (Thread.create Io.load_dir ());
    try
      while true do
        View.wait_action key Automaton.toggle
      done
    with Exit ->
      View.close ();
      let world = WorldHistory.get_state () in
        if world <> WorldHistory.empty then
          Io.save "current.l" world;
        View.save_state ()