open Useful

let png_load name = Png.load ("images/" ^ name ^ ".png") []

let load_images n =
  let img j = png_load (Colour.colour_name.(j)^(string_of_int n)) in
  let a = Array.create 4 (img 0) in
    for i = 1 to 3 do
      a.(i) <- img i
    done;
    a

let some_star s =
  try
    ignore (String.index s '*');
    true
  with
    Not_found -> false  

type file_format = Absolute | Relative | Picture

let load file_name =
  let chan = open_in file_name in
  let loaded =
    (try
      let acc = ref WorldHistory.empty in
      let accumulate c =
        acc := Iv2Set.add c !acc in
        (try
          let mode = ref None in
          let line_counter = ref 0 in
          let pattern_line l =
            for i = 0 to String.length l-1 do
              if String.get l i = '*' then
                accumulate (i, !line_counter)
            done;
            decr line_counter in
          while true do
            let l = input_line chan in
              if String.length l > 0 then(
                if String.get l 0 = '#' then(
                  if String.length l > 1 then(
                    match String.get l 1 with
                      'A' -> mode := Some Absolute
                    | 'R' -> mode := Some Relative
                    | 'P' -> mode := Some Picture
                    | _ -> ())                  
                )else
                  if some_star l || !mode = Some Picture then(
                    match !mode with
                      None ->
                        mode := Some Picture;
                        pattern_line l
                    | Some Picture ->
                        pattern_line l                    
                    | _ ->
                      raise Exit
                  )else
                    let l = left_trim l in
                      if String.length l > 0 then(
                        match String.get l 0 with
                          '#' ->
                            () (* ##... : comment. *)
                        | '0' .. '9' | '-' ->
                          (match !mode with
                            None | Some Absolute | Some Relative ->
                              accumulate 
                                (Scanf.sscanf l "%d %d" Couple.couple)
                          | _ ->
                            raise Exit)
                        | _ ->
                            raise Exit)
                      )
          done
        with End_of_file -> ());
        Some !acc
      with
        Exit -> None) in
    close_in chan;
    loaded

let try_load f file_name =
  match load file_name with
    None -> ()
  | Some pattern -> f pattern

let save file_name world =
  let chan = open_out file_name in
    Printf.fprintf chan "#R\n";
    Iv2Set.iter (Couple.uncurry2 (Printf.fprintf chan "%i %i\n")) world;
    close_out chan

let read_dir_files path =
  let dir_handle = Unix.opendir path in
  let acc = ref [] in
    (try
       while true do
         match Unix.readdir dir_handle with
           "." | ".." -> ()
         | filename -> let pathname = Filename.concat path filename in
                         if (Unix.lstat pathname).Unix.st_kind
                         = Unix.S_REG then
                           acc := filename::!acc
       done
     with
       End_of_file -> ());
     Unix.closedir dir_handle;
     !acc 

let patterns = ref None
      
let load_dir () =
  let names = List.filter (string_end ".l") (read_dir_files ".") in
  let n = List.length names in
  let side = 2*((int_of_float (sqrt (float_of_int (n-1)))+1)/2)+1 in
  let p = Array.make_matrix side side ("", Iv2Set.empty) in
  let i = ref 0 in
  let j = ref 0 in
    List.iter
      (fun name ->
        try_load
          (fun pattern ->
             p.(!i).(!j) <- name, pattern;
             incr i;
             if !i = side then(
               i := 0;
               incr j
             )) name)
      names;
    patterns := Some p